@@ -60,6 +60,7 @@ char *Hostname = NULL;
 char *InterfaceAddress = NULL;
 char  LocalHostname[128];
 int   dns = 1;
+int   enablempls = 0;
 int   cpacketsize = 64;          /* default packet size */
 int   bitpattern = 0;
 int   tos = 0;
@@ -141,6 +142,7 @@ void parse_arg (int argc, char **argv)
 				   overload psize<0, ->rand(min,max) */
     { "bitpattern", 1, 0, 'b' },/* overload b>255, ->rand(0,255) */
     { "tos", 1, 0, 'Q' },	/* typeof service (0,255) */
+    { "mpls", 0, 0, 'e' },
     { "no-dns", 0, 0, 'n' },
     { "address", 1, 0, 'a' },
     { "first-ttl", 1, 0, 'f' },	/* -f & -m are borrowed from traceroute */
@@ -155,7 +157,7 @@ void parse_arg (int argc, char **argv)
   while(1) {
     /* added f:m:o: byMin */
     opt = getopt_long(argc, argv,
-		      "vhrwxtglpo:i:c:s:b:Q:na:f:m:u46", long_options, NULL);
+		      "vhrwxtglpo:i:c:s:b:Q:ena:f:m:u46", long_options, NULL);
     if(opt == -1)
       break;
 
@@ -199,6 +201,9 @@ void parse_arg (int argc, char **argv)
     case 'a':
       InterfaceAddress = optarg;
       break;
+    case 'e':
+      enablempls = 1;
+      break;
     case 'n':
       dns = 0;
       break;
@@ -377,7 +382,7 @@ int main(int argc, char **argv)
   if (PrintHelp) {
     printf("usage: %s [-hvrwctglspniu46] [--help] [--version] [--report]\n"
 	   "\t\t[--report-wide] [--report-cycles=COUNT] [--curses] [--gtk]\n"
-           "\t\t[--raw] [--split] [--no-dns] [--address interface]\n" /* BL */
+           "\t\t[--raw] [--split] [--mpls] [--no-dns] [--address interface]\n" /* BL */
            "\t\t[--psize=bytes/-s bytes]\n"            /* ok */
            "\t\t[--report-wide|-w] [-u]\n"            /* rew */
 	   "\t\t[--interval=SECONDS] HOSTNAME [PACKETSIZE]\n", argv[0]);
