@@ -52,7 +52,8 @@ void report_open(void)
 
 void report_close(void) 
 {
-  int i, j, at, max, z, w;
+  int i, j, k, at, max, z, w;
+  struct mplslen *mpls, *mplss;
   ip_t *addr;
   ip_t *addr2 = NULL;  
   char name[81];
@@ -102,6 +103,7 @@ void report_close(void)
   at  = net_min();
   for(; at < max; at++) {
     addr = net_addr(at);
+    mpls = net_mpls(at);
     if( addrcmp( (void *) addr, (void *) &unspec_addr, af ) == 0 ) {
       sprintf(name, "???");
     } else {
@@ -138,24 +140,47 @@ void report_close(void)
      * This feature show 'loadbalances' on routes 
      */
     
-   /* z is starting at 1 because addrs[0] is the same that addr */
+    /* z is starting at 1 because addrs[0] is the same that addr */
     for (z = 1; z < MAXPATH ; z++) {
-       addr2 = net_addrs(at, z);
-       int found = 0;
-       if ((addrcmp ((void *) &unspec_addr, (void *) addr2, af)) == 0)
-          break;
+      addr2 = net_addrs(at, z);
+      mplss = net_mplss(at, z);
+      int found = 0;
+      if ((addrcmp ((void *) &unspec_addr, (void *) addr2, af)) == 0)
+        break;
       for (w = 0; w < z; w++)
         /* Thales -- Ok... checking if there are ips repeated on same hop */
-           if ((addrcmp ((void *) addr2, (void *) net_addrs (at,w), af)) == 0) {
-              found = 1;
-              break;
-           }   
-       if (!found) {
-        if (z == 1)  
+        if ((addrcmp ((void *) addr2, (void *) net_addrs (at,w), af)) == 0) {
+           found = 1;
+           break;
+        }   
+
+      if (!found) {
+  
+        if (mpls->labels && z == 1 && enablempls) {
+          for (k=0; k < mpls->labels; k++) {
+            printf("    |  |+-- [MPLS: Lbl %lu Exp %u S %u TTL %u]\n", mpls->label[k], mpls->exp[k], mpls->s[k], mpls->ttl[k]);
+          }
+        }
+
+        if (z == 1) {
           printf ("    |  `|-- %s\n", strlongip(addr2));
-        else 
+          for (k=0; k < mplss->labels && enablempls; k++) {
+            printf("    |   +-- [MPLS: Lbl %lu Exp %u S %u TTL %u]\n", mplss->label[k], mplss->exp[k], mplss->s[k], mplss->ttl[k]);
+          }
+        } else {
           printf ("    |   |-- %s\n", strlongip(addr2));
-       }
+          for (k=0; k < mplss->labels && enablempls; k++) {
+            printf("    |   +-- [MPLS: Lbl %lu Exp %u S %u TTL %u]\n", mplss->label[k], mplss->exp[k], mplss->s[k], mplss->ttl[k]);
+          }
+        }
+      }
+    }
+
+    /* No multipath */
+    if(mpls->labels && z == 1 && enablempls) {
+      for (k=0; k < mpls->labels; k++) {
+        printf("    |   +-- [MPLS: Lbl %lu Exp %u S %u TTL %u]\n", mpls->label[k], mpls->exp[k], mpls->s[k], mpls->ttl[k]);
+      }
     }
   }
 }
